"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class dial1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'dial1' service.
    """

    SERVICE_MANUFACTURER = 'MicrosoftCorporation'
    SERVICE_TYPE = 'urn:dial-multiscreen-org:service:dial:1'

    SERVICE_EVENT_VARIABLES = {}
