"""
.. module:: clients
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: The clients package contains couplings that are utilized to connect
               tests with the requirements and functionality to interoperate with
               linux and windows clients.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"
